"""Visitor functions."""

# Standard imports
import ast
from itertools import product
from linecache import getline
from pathlib import Path
from typing import Any, cast

# Local imports
from .classes import IssueEnum, IssueInfo


def get_attr(obj: object, name: str, default: Any | None = None) -> Any:  # noqa: ANN401
    """
    Get attribute of an object.

    The function is a implementation of `dict().get` for any object.

    Args:
        obj: Object from which we want the given attribute.
        name: Attribute we want.
        default: Default value return if the object don't have the attribute.

    Returns:
        Value of the attribute if the object have it, the default otherwise.

    Examples:
        >>> from dataclasses import dataclass
        >>> @dataclass
        >>> class Toto():
        ...     titi: int
        ...     tutu: int | None = None

        >>> toto = Toto(titi = 1, tutu = 2)
        >>> get_attr(toto, "titi")
        1
        >>> get_attr(toto, "tutu")
        2

        >>> toto2 = Toto(titi = 1)
        >>> get_attr(toto2, "titi")
        1
        >>> get_attr(toto2, "tutu")
        None
        >>> get_attr(toto2, "foo")
        None
        >>> get_attr(toto2, "foo", 3)
        3
    """
    if hasattr(obj, name):
        return getattr(obj, name)

    return default


def get_display_func_or_none(node: ast.Expr) -> str | None:
    """
    Get a display function in a ast.Expr.

    Function detected by this one:
        - sys.stdout.write()
        - stdout.write()
        - sys.stderr.write()
        - stderr.write()
        - sys.stdout.writelines()
        - stdout.writelines()
        - sys.stderr.writelines()
        - stderr.writelines()

    Args:
        node: Expr to parse may can contains one of function we search.

    Returns:
        The function name if we find a function of the list, None otherwizse.
    """
    std_output = ["stdout", "stderr"]
    std_function = ["write", "writelines"]

    for res in product(std_output, std_function):
        output = res[0]
        funct = res[1]
        # Detect:
        # - sys.stdout.write
        # - sys.stderr.write
        # - sys.stdout.writelines
        # - sys.stderr.writelines
        try:
            if (
                node.value.func.value.value.id == "sys"  # type: ignore[attr-defined]
                and node.value.func.value.attr == output  # type: ignore[attr-defined]
                and node.value.func.attr == funct  # type: ignore[attr-defined]
            ):
                return f"sys.{output}.{funct}"
        except AttributeError as err:  # noqa: F841 <Only used for debug>
            pass

        # Detect:
        # - stdout.write
        # - stderr.write
        # - stdout.writelines
        # - stderr.writelines
        try:
            if (
                node.value.func.value.id == output  # type: ignore[attr-defined]
                and node.value.func.attr == funct  # type: ignore[attr-defined]
            ):
                return f"{output}.{funct}"
        except AttributeError as err:  # noqa: F841 <Only used for debug>
            pass

    return None


class PrintNodeVisitor(ast.NodeVisitor):
    """
    Print visitor class.

    Attributes:
        found_prints: All issues for print found.

    """

    found_prints: list[IssueInfo] = []

    def visit_Expr(self, node: ast.Expr) -> None:  # noqa: N802
        """
        Visitor expression method.

        Args:
            node: Node to visit.

        Returns:
            Issue if we find one, otherwise return None.
        """
        if not isinstance(node.value, ast.Call):  # pragma: no cover
            return None

        func = node.value.func
        match get_attr(func, "id", None):
            case "print":
                issue_type = IssueEnum.PRINTDETECT
            case "pprint":
                issue_type = IssueEnum.PRETTYPRINTDETECT
            case None:
                # Function not directly call like: `sys.stdout.write`
                detected_func_or_none = get_display_func_or_none(node)
                match detected_func_or_none:
                    case "sys.stdout.write" | "stdout.write":
                        issue_type = IssueEnum.SYSSTDOUTWRITEDETECT
                    case "sys.stderr.write" | "stderr.write":
                        issue_type = IssueEnum.SYSSTDERRWRITEDETECT
                    case "sys.stdout.writelines" | "stdout.writelines":
                        issue_type = IssueEnum.SYSSTDOUTWRITELINESDETECT
                    case "sys.stderr.writelines" | "stderr.writelines":
                        issue_type = IssueEnum.SYSSTDERRWRITELINESDETECT
                    case _:
                        return None
            case _:
                # Function we wish to ignore
                return None

        found_print = IssueInfo(
            issue_type,
            node.lineno,
            node.col_offset,
            line_as_str=None,
            from_file=None,
            ignored=False,
        )
        self.found_prints.append(cast(IssueInfo, found_print))

        return None


def _add_file_info(node_visitor: PrintNodeVisitor, file_path: Path) -> None:
    """
    Add information of file in issues.

    Args:
        node_visitor: Node visitor.
        file_path: Path of the file that we are analyzing.
    """
    for i in range(len(node_visitor.found_prints)):
        if node_visitor.found_prints[i].from_file is None:
            node_visitor.found_prints[i].from_file = file_path
            node_visitor.found_prints[i].line_as_str = getline(
                str(file_path),
                node_visitor.found_prints[i].num_line,
            ).strip()


def contains_print(file_path: Path, tree: ast.AST) -> list[IssueInfo]:
    """
    Detect if a given file contains print.

    Args:
        file_path: Path of the given file.
        tree: AST of the file.

    Returns:
        All print found.
    """
    node_visitor = PrintNodeVisitor()
    node_visitor.visit(tree)
    _add_file_info(node_visitor, file_path)
    return node_visitor.found_prints



"""Issues classes used in the project."""

# Standard imports
from collections import namedtuple
from dataclasses import dataclass
from enum import Enum
from pathlib import Path

Issue = namedtuple("Issue", ["err_code", "name"])


class IssueEnum(Issue, Enum):
    """Issue Enumerable to store issues."""

    PRINTDETECT = Issue(err_code="PPL001", name="print-detected")
    PRETTYPRINTDETECT = Issue(err_code="PPL002", name="prettyprint-detected")
    SYSSTDOUTWRITEDETECT = Issue(err_code="PPL003", name="sys.stdout.write-detected")
    SYSSTDERRWRITEDETECT = Issue(err_code="PPL004", name="sys.stderr.write-detected")
    SYSSTDOUTWRITELINESDETECT = Issue(
        err_code="PPL005", name="sys.stdout.writelines-detected"
    )
    SYSSTDERRWRITELINESDETECT = Issue(
        err_code="PPL006", name="sys.stderr.writelines-detected"
    )


@dataclass
class IssueInfo:
    """Issue information class."""

    issue: IssueEnum
    "Issue type."

    num_line: int
    "Line number of the issue."

    num_col: int
    "Column number of the issue."

    line_as_str: str | None
    "String representation of the line."

    from_file: Path | None
    "File who contain the line with the issue."

    ignored: bool
    "If this issue is ignored or not."

    def __str__(self) -> str:
        """
        Get issue as string.

        Returns:
            The string representation of the issue.
        """
        return (
            f"[bold]{self.from_file}[/bold]:{self.num_line}:{self.num_col}: "
            f"[bold red]{self.issue.err_code}[/bold red] [bold]`[/bold]"
            f"{self.line_as_str}[bold]`[/bold] {self.issue.name}"
        )


@dataclass
class IgnoredLine:
    """Ignored line class."""

    line_num: int
    "Line number of the ignored line."

    error_code: str
    "Code of the ignored issue."

    from_file: Path | None
    "File who contains the ignored line."


@dataclass
class IgnoredFile:
    """Ignored file class."""

    error_code: str
    "Code of the ignored issue."

    from_file: Path | None
    "File who contains the ignored line."


@dataclass
class IgnoredBlock:
    """Ignored block of code class."""

    error_code: str
    "Code of the ignored issue."

    line_from: int
    "Begin of ignored block."

    line_to: int
    "End of ignored block."

    from_file: Path | None
    "File who contains the ignored line."

    def __init__(
        self,
        error_code: str,
        line_from: int,
        line_to: int,
        from_file: Path | None,
    ) -> None:
        """
        Initialize an IngoredBlock class.

        Args:
            error_code: Given error code.
            line_from: Given line from.
            line_to: Given line to.
            from_file: Given from file.

        Raise:
            ValueError: If `g_line_from` <= `g_line_to`.
        """
        if line_to <= line_from:
            raise ValueError(
                f"Given line to ({line_to}) must be strictly greater then given line "
                f"from ({line_from})"
            )

        self.error_code = error_code
        self.line_from = line_from
        self.line_to = line_to
        self.from_file = from_file



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['cli_app', 'printlinter']

package_data = \
{'': ['*']}

install_requires = \
['pyyaml>=6.0.1,<7.0.0',
 'rich>=12.5.1,<13.0.0',
 'toml>=0.10.2,<0.11.0',
 'typer>=0.7.0,<0.8.0']

entry_points = \
{'console_scripts': ['printlinter = cli_app.main:main']}

setup_kwargs = {
    'name': 'printlinter',
    'version': '0.2.0',
    'description': 'print linter to know where is the test print',
    'long_description': '<!-- markdownlint-disable-file MD041 -->\n\n<!-- TODO: Add badge -->\n![logo](logo.png)\n\n[English readme](README.md) **·** [Français readme](doc/readme/README.fr.md)\n\nPrintLinter is a python linter to detect and signals display functions in python code.\n\n## Summary\n\n- [Installing](#installing)\n- [Usage](#usage)\n  - [Verbs](#verbs)\n    - [Lint](#lint)\n      - [Example](#example)\n- [Error codes](#error-codes)\n  - [Standard library](#standard-library)\n- [Ignore errors](#ignore-errors)\n  - [Ignore error inline](#ignore-error-inline)\n    - [Examples](#examples)\n  - [Ignore error on the next line](#ignore-error-on-the-next-line)\n    - [Examples](#examples-1)\n  - [Ignore a whole file](#ignore-a-whole-file)\n    - [Examples](#examples-2)\n      - [Simple error](#simple-error)\n      - [library errors](#library-errors)\n      - [All errors](#all-errors)\n  - [Ignore a block of code](#ignore-a-block-of-code)\n    - [Examples](#examples-3)\n      - [Simple error](#simple-error-1)\n      - [All errors](#all-errors-1)\n- [Configuration](#configuration)\n  - [Target version](#target-version)\n  - [Ignored files](#ignored-files)\n  - [Disabled rules](#disabled-rules)\n  - [Color](#color)\n  - [Examples of configuration files](#examples-of-configuration-files)\n    - [Yaml/Yml configuration file](#yamlyml-configuration-file)\n    - [Json configuration file](#json-configuration-file)\n    - [Toml configuration file](#toml-configuration-file)\n    - [Pyproject configuration file](#pyproject-configuration-file)\n- [But then](#but-then)\n\n# Installing\n\nInstall with `pip` or your favorite PyPI package manageer.\n\n```sh\npip install printlinter\n```\n\nRun the following command to test PrintLinter.\n\n```sh\nprintlinter --version\n```\n\n# Usage\n\n## Verbs\n\nFor help on using a verb.\n\n```sh\nprintlinter <verb> --help\n```\n\n### Lint\n\nTo lint a file or a folder.\n\n```sh\nprintlinter lint <file|folder>\n```\n\nThe product output look like this.\n\n`file_path:line:column error_code display_function_detected error_name`\n\nBy default we disable the linter in some folders.\n\n- `node_modules`\n- `.vscode/`\n- `__pycache__/`\n- `build/`\n- `develop-eggs/`\n- `dist/`\n- `downloads/`\n- `eggs/`\n- `.eggs/`\n- `lib/`\n- `lib64/`\n- `parts/`\n- `sdist/`\n- `var/`\n- `wheels/`\n- `pip-wheel-metadata/`\n- `share/python-wheels/`\n- `htmlcov/`\n- `.tox/`\n- `.nox/`\n- `.hypothesis/`\n- `.pytest_cache/`\n- `docs/_build/`\n- `__pypackages__/`\n- `.mypy_cache/`\n- `.ruff_cache`\n- `.pyre/`\n- `env/`\n- `venv/`\n- `ENV/`\n- `env.bak/`\n- `venv.bak/`\n- `.venv/`\n- `.env/`\n\n#### Example\n\n<!-- markdownlint-disable MD013 -->\n```sh\ntests/testing_files/mixed/mixed0.py:5:0: PPL001 `print("toto")` print-detected\ntests/testing_files/mixed/mixed0.py:6:0: PPL002 `pprint("titi")` prettyprint-detected\ntests/testing_files/mixed/mixed1/mixed2.py:5:0: PPL001 `print("toto")` print-detected\ntests/testing_files/mixed/mixed1/mixed2.py:6:0: PPL002 `pprint("titi")` prettyprint-detected\ntests/testing_files/pprint/pprint1.py:6:0: PPL002 `pprint("Hello, world")` prettyprint-detected\ntests/testing_files/pprint/pprint2/pprint3.py:11:4: PPL002 `pprint("tata")` prettyprint-detected\ntests/testing_files/ignored_files/ignore_ppl005.py:7:0: PPL001 `print("toto")` print-detected\ntests/testing_files/ignored_files/ignore_nothing.py:6:0: PPL001 `print("toto")` print-detected\ntests/testing_files/ignored_files/ignore_nothing.py:8:0: PPL002 `pprint("toto")` prettyprint-detected\ntests/testing_files/ignored_files/ignore_ppl006.py:7:0: PPL001 `print("toto")` print-detected\ntests/testing_files/ignored_files/ignore_ppl002.py:7:0: PPL001 `print("toto")` print-detected\ntests/testing_files/ignored_files/ignore_ppl001.py:9:0: PPL002 `pprint("toto")` prettyprint-detected\ntests/testing_files/ignored_files/ignore_ppl004.py:7:0: PPL001 `print("toto")` print-detected\ntests/testing_files/ignored_files/ignore_ppl003.py:7:0: PPL001 `print("toto")` print-detected\ntests/testing_files/ignored_files/disable_in_wrong_place.py:8:0: PPL001 `print("toto")` print-detected\ntests/testing_files/sys/stderr/write/stderr1.py:7:0: PPL004 `sys.stderr.write("Hello, world")` sys.stderr.write-detected\ntests/testing_files/sys/stderr/write/stderr1.py:8:0: PPL004 `stderr.write("Hello, world")` sys.stderr.write-detected\ntests/testing_files/sys/stderr/write/stderr2/stderr3.py:13:4: PPL004 `sys.stderr.write("tata")` sys.stderr.write-detected\ntests/testing_files/sys/stderr/write/stderr2/stderr3.py:14:4: PPL004 `stderr.write("tata")` sys.stderr.write-detected\ntests/testing_files/sys/stderr/writelines/stderr1.py:7:0: PPL006 `sys.stderr.writelines(["Hello", "world"])` sys.stderr.writelines-detected\ntests/testing_files/sys/stderr/writelines/stderr1.py:8:0: PPL006 `stderr.writelines(["Hello", "world"])` sys.stderr.writelines-detected\ntests/testing_files/sys/stderr/writelines/stderr2/stderr3.py:13:4: PPL006 `sys.stderr.writelines(["tata", "tutu"])`\nsys.stderr.writelines-detected\ntests/testing_files/sys/stderr/writelines/stderr2/stderr3.py:14:4: PPL006 `stderr.writelines(["tata", "tutu"])`\nsys.stderr.writelines-detected\ntests/testing_files/sys/stdout/write/stdout1.py:7:0: PPL003 `sys.stdout.write("Hello, world")` sys.stdout.write-detected\ntests/testing_files/sys/stdout/write/stdout2/stdout3.py:13:4: PPL003 `sys.stdout.write("tata")` sys.stdout.write-detected\ntests/testing_files/sys/stdout/writelines/stdout1.py:7:0: PPL005 `sys.stdout.writelines(["Hello", "world"])` sys.stdout.writelines-detected\ntests/testing_files/sys/stdout/writelines/stdout2/stdout3.py:14:4: PPL005 `stdout.writelines(["tata", "tutu"])`\nsys.stdout.writelines-detected\ntests/testing_files/print/toto_1.py:3:0: PPL001 `print("Hello, world")` print-detected\ntests/testing_files/print/toto2/toto3.py:7:4: PPL001 `print("tata")` print-detected\nFound 27 errors\n```\n<!-- markdownlint-enable MD013 -->\n\n# Error codes\n\nAll errors have an individual code based on `PPLXXX`.\nEach library have its own error "domain".\n\n| Library  | Error domain |\n|:---------|:-------------|\n| Standard | PPL0XX       |\n\n## Standard library\n\n| Function                | Error code |\n|:------------------------|:-----------|\n| `print`                 | PPL001     |\n| `pprint`                | PPL002     |\n| `sys.stdout.write`      | PPL003     |\n| `sys.stderr.write`      | PPL004     |\n| `sys.stdout.writelines` | PPL005     |\n| `sys.stderr.writelines` | PPL006     |\n\n# Ignore errors\n\nYou can ignore errors with the linter.\n\n## Ignore error inline\n\nTo ignore a rule inline, add a comment at the end of the line. `# noqa: <error_code>`.\n\n### Examples\n\n```python\ntoto = 1\ntiti = 2\nprint (toto + titi) # noqa: PPL001\n```\n\n## Ignore error on the next line\n\nTo ignore a rule on the next line, add a comment at the line before code to ignore.\n`<printlinter disable-next (error_code)>`.\n\n<!-- markdownlint-disable-next-line MD024 -->\n### Examples\n\n```python\ntoto = 1\ntiti = 2\n# <printlinter disable-next PPL001>\nprint (toto + titi) # ignored error\n```\n\n## Ignore a whole file\n\nTo ignore a rule, library rules or all rules, add a comment at the beging of the file.\n`# <printlinter disable-file (error_code)>`.\n\n<!-- markdownlint-disable-next-line MD036 -->\n**The comment must be before any code in a file.**\n\n<!-- markdownlint-disable-next-line MD024 -->\n### Examples\n\n#### Simple error\n\n```python\n# <printlinter disable-file PPL002>\nfrom pprint import pprint\ntoto = 1\ntiti = 2\npprint(titi + toto)  # ignored error\n...\n```\n\n#### library errors\n\n```python\n# <printlinter disable-file PPL000>\nimport sys\ntoto = 1\ntiti = 2\nprint(titi + toto)  # ignored error\nsys.stdout.write(titi + toto)  # ignored error\n...\n```\n\n#### All errors\n\n```python\n# <printlinter disable-file ALL>\nfrom sys import stdout, stderr\nfrom pprint import pprint\ntoto = 1\ntiti = 2\nprint(titi + toto)  # ignored error\npprint(titi + toto)  # ignored error\nstdout.write(titi + toto)  # ignored error\nstderr.write(titi + toto)  # ignored error\nstdout.writelines(titi + toto)  # ignored error\nstderr.writelines(titi + toto)  # ignored error\n...\n```\n\n## Ignore a block of code\n\nTo ignore a block of code (disable the linter) on a rules or all rules (disable library\nrules come later), add a comment before the block you want ignore:\n`<printlinter disable (error_code)`. To re enable the linter add an other comment after\nthe block of code: `<printlinter enable (error_code)>`. You are don\'t have to re enable\nthe linter after disabled it.\n\n<!-- markdownlint-disable-next-line MD024 -->\n### Examples\n\n<!-- markdownlint-disable-next-line MD024 -->\n#### Simple error\n\n```python\nfrom pprint import pprint\n# <printlinter disable PPL001>\nprint("toto") # ignored error\npprint("titit")\n# <printlinter enable PPL001>\n...\n```\n\nYou can nested ignored block.\n\n```python\nfrom pprint import pprint\n\n# <printlinter disable PPL002>\n\npprint("titi") # ignored error\n\n# <printlinter disable PPL001>\nprint("toto") # ignored error\n# <printlinter enable PPL001>\n\nprint("tutu") # NOT IGNORED ERROR\npprint("tata") # ignored error\n\n# <printlinter enable PPL002>\n...\n```\n\nwithout re enable the linter.\n\n```python\nfrom pprint import pprint\n# <printlinter disable PPL001>\nprint("toto") # ignored error\npprint("titit")\n...\nprint(\'toto\') # ignored error\n```\n\n<!-- markdownlint-disable-next-line MD024 -->\n#### All errors\n\n```python\nfrom sys import stdout, stderr\nfrom pprint import pprint\ntoto = 1\ntiti = 2\n# <printlinter disable ALL>\nprint(titi + toto)  # ignored error\npprint(titi + toto)  # ignored error\nstdout.write(titi + toto)  # ignored error\nstderr.write(titi + toto)  # ignored error\nstdout.writelines(titi + toto)  # ignored error\nstderr.writelines(titi + toto)  # ignored error\n# <printlinter enable ALL>\n...\n```\n\n# Configuration\n\nYou can configurate the linter with a configuration file (configuration by command line,\nwill arrive in a future version).\n\nPrintLinter supports 3 differents file type.\n\n- `.yaml/.yml`\n- `.json`\n- `.toml`\n\nBy default the linter use one of thoses files load in this order.\n\n- `printlinter.yaml`\n- `printlinter.yml`\n- `printlinter.json`\n- `printlinter.toml`\n- `pyproject.toml`\n\nAlternatively, you can use another file and fill it in by running linter from the command\nline.\n\n```sh\nprintlinter lint <file|folder> --config-file </path/of/config/file>\n```\n\n## Target version\n\nThe config file allows to specify the python target version for the parser. Use a\nstring like this `3.10` or `3.7` to specify the version.\n\n<!-- markdownlint-disable-next-line MD036 -->\n**The python target version MUST be contains between 3.7 and 3.10**\n\nYou can see examples of this config [here](#examples-of-configuration-files).\n\n## Ignored files\n\nThe config file allows to ignore files, unlike the comment that allows errors to be\nignored in a whole file this configuration option prevents the linter from reading the file.\n\nYou can see examples of this config [here](#examples-of-configuration-files).\n\n## Disabled rules\n\nThe config file allows to disable rules, in all files and folders.\n\nYou can see examples of this config [here](#examples-of-configuration-files).\n\n## Color\n\nThe config file allows to enable or disable colorized output. By default the `color` option\nis on `True`.\n\nYou can see examples of this config [here](#examples-of-configuration-files).\n\n## Examples of configuration files\n\n### Yaml/Yml configuration file\n\n```yaml\nprintlinter:\n  target_version: "3.10"\n  ignored_files: [toto.py]\n  disabled_rules: [PPL001]\n  color: true\n```\n\n### Json configuration file\n\n```json\n{\n  "printlinter": {\n    "target_version": "3.10",\n    "ignored_files": ["toto.py"],\n    "disabled_rules": ["PPL001"],\n    "color": false\n  }\n}\n\n```\n\n### Toml configuration file\n\n```toml\n[printlinter]\ntarget_version = "3.10"\nignored_files = ["toto.py"]\ndisabled_rules = ["PPL001"]\ncolor = true\n```\n\n### Pyproject configuration file\n\n```toml\n[tool.printlinter]\ntarget_version = "3.10"\nignored_files = ["toto.py"]\ndisabled_rules = ["PPL001"]\ncolor = false\n```\n\n# But then\n\nToo see the next features to be developed see [TODO](TODO.md). Here\'s a small,\nnon-exhaustive list of what\'s comming in the future versions.\n\n- Lint display functions from other libraries.\n- Add other configuration options.\n',
    'author': 'ulysse',
    'author_email': 'ulysse.chosson@obspm.fr',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'None',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.10,<4.0',
}


setup(**setup_kwargs)



"""
PrintLinter is a program to lint python project and detect trailling display functions.

It is a TYPER program that can laucnh the different verbs (actions) with their
respective options.
"""

# First party imports
from cli_app import APP


def main() -> None:
    """Launch cli application for printlinter."""
    APP()


if __name__ == "__main__":
    main()



"""Processing ingoed lines functions."""

# Local imports
from .classes import IgnoredBlock, IgnoredFile, IgnoredLine, IssueInfo

FAMILIRY_ERR_CODE = [
    "PPL000",
    "PPL100",
    "PPL200",
    "PPL300",
    "PPL400",
    "PPL500",
    "PPL600",
    "PPL700",
    "PPL800",
    "PPL900",
]


def get_not_ignored_issue(
    issues: list[IssueInfo],
    ignored_lines: list[IgnoredLine],
    ignored_files: list[IgnoredFile],
    ignored_blocks: list[IgnoredBlock],
    disabled_rules: list[str],
) -> list[IssueInfo]:
    """
    Get all not ignored issues.

    Args:
        issues: All issue found.
        ignored_lines: All ignored lines.
        ignored_files: All ignored files.
        ignored_blocks: All ignored blocks of code.
        disabled_rules: All disabled rules.

    Returns:
        All lines are not ignored.
    """
    result = []

    for issue in issues:
        if issue.issue.err_code in disabled_rules:
            continue

        ignored_all_file_equivalence = IgnoredFile(
            error_code="ALL",
            from_file=issue.from_file,
        )
        if ignored_all_file_equivalence in ignored_files:
            continue

        # Ignore a file with a family of code (like PPL000, PPL100, ...)
        for index in range(len(FAMILIRY_ERR_CODE) - 1):
            err_code_min = FAMILIRY_ERR_CODE[index]
            err_code_max = FAMILIRY_ERR_CODE[index + 1]
            if err_code_min < issue.issue.err_code < err_code_max:
                ignored_err_code = err_code_min

        ignored_family_file_equivalence = IgnoredFile(
            error_code=ignored_err_code,
            from_file=issue.from_file,
        )
        if ignored_family_file_equivalence in ignored_files:
            continue

        # Files with specific code (like PPL001, PPL005, ...)
        ignored_file_equivalence = IgnoredFile(
            error_code=issue.issue.err_code,
            from_file=issue.from_file,
        )
        if ignored_file_equivalence in ignored_files:
            continue

        # Ignored lines
        ignored_line_equivalence = IgnoredLine(
            issue.num_line,
            error_code=issue.issue.err_code,
            from_file=issue.from_file,
        )
        if ignored_line_equivalence in ignored_lines:
            continue

        # Ignored blocks
        # TODO: ignore a librairy rules for a block of code
        issue_in_ignored_block = False
        for ignored_block in ignored_blocks:
            if (
                ignored_block.from_file == issue.from_file
                and ignored_block.line_from <= issue.num_line <= ignored_block.line_to
                and (
                    ignored_block.error_code == "ALL"
                    or ignored_block.error_code == issue.issue.err_code
                )
            ):
                issue_in_ignored_block = True
                continue

        if issue_in_ignored_block:
            continue

        result.append(issue)

    return result
